/*
  examplePosControl_MEGA
  Drives HerkuleX servos thanks to the HerkuleXLib (see 
  www.benoitpuel.com/blog/telechargements/) using an Arduino MEGA board.

  The serial terminal is used to printout messages.

  This example code is in the public domain.

  modified 10 February 2016
  by Benoit Puel
 */

#include <HkxPosControl.h>

void setup() {
  // INPUT
  byte ID = 0;
  
  delay(5000); // wait 5 seconds to open the serial monitor
  HkxPrint printoutLib = HkxPrint(Serial, 9600);  // Printout errors on the serial monitor
  HkxPrint printoutCode = HkxPrint(printoutLib, true, true, true);  // Printout errors on the serial monitor
  HkxCommunication communication = HkxCommunication(HKX_666666, Serial1, printoutLib);  // Communication with the servo on Serial1
  HkxPosControl servo(ID, communication, printoutLib);  // control position for the servo ID

  // Variables
  HkxStatus statusED;
  String message;
  uint16_t voltage;
  uint16_t temperature;
  int16_t position;
  int16_t velocity;
  uint16_t PWM;
  int16_t goalPosition;
  int16_t trajectoryPosition;
  int16_t trajectoryVelocity;

  // Generate a packet error
  byte errorPacket[] = {0xFF, 0xFF, 0x00, 0x22, 0x43, 0xA1, 0x08};
  Serial1.write(errorPacket, 7);
  delay(1000);
  // Check if there is any statu error
  servo.getStatus(statusED, true);
  if(statusED.isError(HKX_STAT_ALL)){
    printoutCode.errorPrint("The servo has an error");
    if(statusED.isError(HKX_STAT_VOLTAGE)){
      printoutCode.errorPrint("Exceed Input Voltage limit");
    } if(statusED.isError(HKX_STAT_POSITION)){
      printoutCode.errorPrint("Exceed allowed POT limit");
    } if(statusED.isError(HKX_STAT_TEMPERATURE)){
      printoutCode.errorPrint("Exceed Temperature limit");
    } if(statusED.isError(HKX_STAT_PACKET)){
      printoutCode.errorPrint("Invalid Packet");
    } if(statusED.isError(HKX_STAT_OVERLOAD)){
      printoutCode.errorPrint("Overload detected");
    } if(statusED.isError(HKX_STAT_DRIVER)){
      printoutCode.errorPrint("Driver fault detected");
    } if(statusED.isError(HKX_STAT_ROM_DISTORTED)){
      printoutCode.errorPrint("EEP REG distorted");
    }
  }
  servo.clearStatus();

  // Set the LED and torque off
  servo.setTorqueLEDControl(HKX_TORQUE_FREE, HKX_LED_OFF);  
  printoutCode.infoPrint("LED is off, torque is free");
  delay(5000); // wait 5 seconds

  // Set the LED to red
  servo.setTorqueLEDControl(HKX_NO_VALUE, HKX_LED_RED);  
  printoutCode.infoPrint("LED is now red");
  delay(5000); // wait 5 seconds

  // Set the servo to position 45° in 2 seconds, led turns to blue during the move, 
  // wait for the end of the move
  printoutCode.infoPrint("Start moving to 45 degrees in 2 seconds, blue LED");
  servo.movePosition(450, 2000, HKX_LED_BLUE, true);  
  printoutCode.infoPrint("The move is finished, torque is free and LED back to red");

  // Get the current behaviour of the servo
  printoutCode.infoPrint("Volt \t Temp \t Pos \t Vel \t PWM \t gPos \t tPos \t tVel");
  servo.getBehaviour(&voltage, &temperature, &position, &velocity, &PWM, &goalPosition, &trajectoryPosition, &trajectoryVelocity);
  message = String((float)voltage/1000);
  message += " \t ";
  message += (float)temperature/100;
  message += " \t ";
  message += (float)position/10;
  message += " \t ";
  message += velocity;
  message += " \t ";
  message += PWM;
  message += " \t ";
  message += (float)goalPosition/10;
  message += " \t ";
  message += (float)trajectoryPosition/10;
  message += " \t ";
  message += trajectoryVelocity;
  printoutCode.infoPrint(message);
  delay(5000); // wait 5 seconds

  // Set the servo to position -45° in 2 seconds, led turns to green, 
  // do NOT wait for the end of the move
  printoutCode.infoPrint("Start moving to -45 degrees in 2 seconds, green LED");
  printoutCode.infoPrint("Volt \t Temp \t Pos \t Vel \t PWM \t gPos \t tPos \t tVel");
  servo.movePosition(-450, 2000, HKX_LED_GREEN, false);  
  // Get the current behaviour of the servo during the move
  while(!servo.isInPosition()){
    servo.getBehaviour(&voltage, &temperature, &position, &velocity, &PWM, &goalPosition, &trajectoryPosition, &trajectoryVelocity);
    message = String((float)voltage/1000);
    message += " \t ";
    message += (float)temperature/100;
    message += " \t ";
    message += (float)position/10;
    message += " \t ";
    message += velocity;
    message += " \t ";
    message += PWM;
    message += " \t ";
    message += (float)goalPosition/10;
    message += " \t ";
    message += (float)trajectoryPosition/10;
    message += " \t ";
    message += trajectoryVelocity;
    printoutCode.infoPrint(message);
  }
  printoutCode.infoPrint("The move is finished, torque is keep on and LED stays green");
}

void loop() {
  // put your main code here, to run repeatedly:

}
